import euclid

from OpenGL.GL import *
from OpenGL.GLUT import *

XPOS = euclid.Vector3(1., 0, 0)
XNEG = euclid.Vector3(-1., 0, 0)
YPOS = euclid.Vector3(0, 1., 0)
YNEG = euclid.Vector3(0, -1., 0)
ZPOS = euclid.Vector3(0, 0, 1.)
ZNEG = euclid.Vector3(0, 0, -1.)

class AABox:
    '''Axis-Aligned Box centered on "center" with dimensions x, y, z'''
    def __init__(self, center, dx, dy, dz):
        self.dx, self.dy, self.dz = dx, dy, dz
        self.set_center(center)

    def set_center(self, center):
        c = self.c = euclid.Point3(*center)

        x = (c.x - self.dx/2, c.x + self.dx/2)
        self.xmin = min(x)
        self.xmax = max(x)

        y = (c.y - self.dy/2, c.y + self.dy/2)
        self.ymin = min(y)
        self.ymax = max(y)

        z = (c.z - self.dz/2, c.z + self.dz/2)
        self.zmin = min(z)
        self.zmax = max(z)

    def __repr__(self):
        a = tuple(self.c) + (self.dx, self.dy, self.dz)
        return '<AABox c=%s,%s,%s d=%s,%s,%s>'%a

    def is_intersecting(self, other):
        if isinstance(other, euclid.Point3):
            return (self.xmin < other.x < self.xmax and
                    self.ymin < other.y < self.ymax and
                    self.zmin < other.z < self.zmax)
        elif isinstance(other, AABox):
            c = other.c
            hdx, hdy, hdz = other.dx/2, other.dy/2, other.dz/2
            return (self.xmin-hdx < c.x < self.xmax+hdx and
                self.ymin-hdy < c.y < self.ymax+hdy and
                self.zmin-hdz < c.z < self.zmax+hdz)
        else:
            raise NotImplementedError

    def resolve_collision(self, obj):
        '''An object has been determined to have collided with this object
        by is_intersecting() above. This method is to be called with the
        object in its *pre* moved state, and will move it as far as it can
        such that it touches this object (but this object is considered
        absolutely stationary).
        '''
        other = obj.hitbox
        if not isinstance(other, AABox): raise NotImplementedError
        c = other.c

        # test from middle of closest edges
        s = self.c
        sx = s.x; cx = c.x
        if other.c.x > self.c.x:
            s.x = self.xmax
            c.x = other.xmin
            v = (c - s).normalize()
            x = v.dot(XPOS)
        else:
            s.x = self.xmin
            c.x = other.xmax
            v = (c - s).normalize()
            x = v.dot(XNEG)
        s.x = sx; c.x = cx
        sy = s.y; cy = c.y
        if other.c.y > self.c.y:
            s.y = self.ymax
            c.y = other.ymin
            v = (c - s).normalize()
            y = v.dot(YPOS)
        else:
            s.y = self.ymin
            c.y = other.ymax
            v = (c - s).normalize()
            y = v.dot(YNEG)
        s.y = sy; c.y = cy
        sz = s.z; cz = c.z
        if other.c.z < self.c.z:
            s.z = self.zmax
            c.z = other.zmin
            v = (c - s).normalize()
            z = v.dot(ZPOS)
        else:
            s.z = self.zmin
            c.z = other.zmax
            v = (c - s).normalize()
            z = v.dot(ZNEG)
        s.z = sz; c.z = cz


        resting = False
        if y > x and y > z:
            c.y = self.ymax + other.dy/2
            obj.velocity.y = 0
            resting = True
        elif x > z:
            hdx = other.dx/2
            if c.x > self.c.x: c.x = self.xmax + hdx
            else: c.x = self.xmin - hdx
            obj.velocity.x = 0
        else:
            hdz = other.dz/2
            if c.z > self.c.z: c.z = self.zmax + hdz
            else: c.z = self.zmin - hdz
            obj.velocity.z = 0


        other.set_center(tuple(c))

        return resting

    def render(self):
        glPushMatrix()
        glTranslate(self.c.x, self.c.y, self.c.z)
        glScale(self.dx, self.dy, self.dz)
        glColor(1, .5, .5)
        glutWireCube(1)
        glPopMatrix()



class Sphere(euclid.Sphere):
    def render(self):
        glPushMatrix()
        glTranslate(self.c.x, self.c.y, self.c.z)
        glColor(1, 1, 1)
        glutWireSphere(self.r, 8, 8)
        glPopMatrix()


